#version 120
/* DRAWBUFFERS:56 */

uniform sampler2D texture;

varying vec4 color;
varying vec2 texcoord;

uniform int worldTime;
uniform ivec2 eyeBrightnessSmooth;
uniform float rainStrength;

float night = clamp((worldTime-13000.0)/300.0,0.0,1.0)-clamp((worldTime-22800.0)/200.0,0.0,1.0);
float cavelight = pow(eyeBrightnessSmooth.y / 255.0, 6.0f) * 1.0 + (0.7 + 0.5*night);

vec3 hueShift( vec3 color, float hueAdjust ){

    const vec3  kRGBToYPrime = vec3 (0.299, 0.587, 0.114);
    const vec3  kRGBToI      = vec3 (0.596, -0.275, -0.321);
    const vec3  kRGBToQ      = vec3 (0.212, -0.523, 0.311);

    const vec3  kYIQToR     = vec3 (1.0, 0.956, 0.621);
    const vec3  kYIQToG     = vec3 (1.0, -0.272, -0.647);
    const vec3  kYIQToB     = vec3 (1.0, -1.107, 1.704);

    float   YPrime  = dot (color, kRGBToYPrime);
    float   I       = dot (color, kRGBToI);
    float   Q       = dot (color, kRGBToQ);
    float   hue     = atan (Q, I);
    float   chroma  = sqrt (I * I + Q * Q);

    hue += hueAdjust;

    Q = chroma * sin (hue);
    I = chroma * cos (hue);

    vec3    yIQ   = vec3 (YPrime, I, Q);

    return vec3( dot (yIQ, kYIQToR), dot (yIQ, kYIQToG), dot (yIQ, kYIQToB) );

}

void main() {

	vec4 albedo = texture2D(texture, texcoord.st)*color;

	vec3 lighting = vec3(1.0+ (0.4*rainStrength - 0.4*rainStrength*night));
		lighting /= 0.8 - 0.5*night;
		lighting /= cavelight;

	albedo.rgb = hueShift(pow(albedo.rgb*0.33, lighting), worldTime * 0.1);

	gl_FragData[0] = albedo;
}